@@ -10,7 +10,7 @@
  *
  * ----------------------------------------------------------------------------
  *
- * $Id: index.php,v 1.73 2006/03/10 00:15:34 mindless Exp $
+ * $Id: index.php,v 1.73.2.1 2006/08/08 00:00:00 mindless Exp $
  *
  * Gallery - a web based photo album viewer and editor
  * Copyright (C) 2000-2006 Bharat Mediratta
@@ -301,7 +301,37 @@ function populateDataDirectory($dataBase) {
 	}
     }
 
-    return true;
+    return secureStorageFolder($dataBase);
+}
+
+/**
+ * Secure the storage folder from attempts to access it directly via the web by adding a
+ * .htaccess with a "Deny from all" directive. This won't have any effect on webservers other
+ * than Apache 1.2+ though.
+ * Since we can't reliably tell whether the storage folder is web-accessible or not,
+ * we add this in all cases. It doesn't hurt.
+ * @param string absolute filesystem path to the storage folder.
+ * @return boolean true if the .htaccess file has been created successfully.
+ */
+function secureStorageFolder($dataBase) {
+    $htaccessPath = $dataBase . '.htaccess';
+    $fh = @fopen($htaccessPath, 'w');
+    if ($fh) {
+	$htaccessContents = "DirectoryIndex .htaccess\n" .
+			    "SetHandler Gallery_Security_Do_Not_Remove\n" .
+			    "Options None\n" .
+			    "<IfModule mod_rewrite.c>\n" .
+			    "RewriteEngine off\n" .
+			    "</IfModule>\n" .
+			    "<IfModule mod_access.c>\n" .
+			    "Order allow,deny\n" .
+			    "Deny from all\n" .
+			    "</IfModule>\n";
+	fwrite($fh, $htaccessContents);
+	fclose($fh);
+    }
+
+    return file_exists($htaccessPath);
 }
 
 /* Returns something like https://example.com */
@@ -315,6 +345,16 @@ function getBaseUrl() {
     return sprintf('%s://%s', $protocol, $hostName);
 }
 
+/** Returns the URL to the G2 folder, e.g. http://example.com/gallery2/. */
+function getGalleryDirUrl() {
+    $galleryDir = dirname(dirname(__FILE__));
+    require_once($galleryDir . '/modules/core/classes/GalleryUrlGenerator.class');
+    $urlPath = preg_replace('|^(.*/)install/index.php(?:\?.*)?$|s', '$1',
+			    GalleryUrlGenerator::getCurrentRequestUri());
+
+    return getBaseUrl() . $urlPath;
+}
+
 /**
  * Mini url generator for the installer
  */
