@@ -1,7 +1,7 @@
 /****************************
  * Hybrid theme for Gallery2
  * @author Alan Harder <alan.harder@sun.com>
- * $Revision: 1.3 $ $Date: 2005/08/17 14:46:14 $
+ * $Revision: 1.10 $ $Date: 2005/10/14 18:29:18 $
  */
 
 //Class app
@@ -23,11 +23,18 @@ function app_init() {
   app_body = document.getElementById('hybridMain');
   while (app_body && app_body.tagName != 'BODY') app_body = app_body.parentNode;
   if (app_is_ie) app_body = app_body.parentNode;
-  app_onresize();
   imagearea = document.getElementById('imagearea');
   imagediv = document.getElementById('imagediv');
   textdiv = document.getElementById('textdiv');
 
+  // Replace <object> (XHTML compliant) with <iframe>
+  // Currently <object> works only with Mozilla and Opera.  IE doesn't accept
+  // any object.data changes, Safari only accepts new data src when object is
+  // visible, Firefox only when object is invisible.  Easier to just use iframe.
+  var popup = document.getElementById('popup_details'), iframe = document.createElement('iframe');
+  iframe.frameBorder = 0;
+  popup.replaceChild(iframe, popup.firstChild);
+
   document.onkeypress = app_onkeypress;
   if (window.attachEvent) window.attachEvent("onresize", app_onresize);
   else if (window.addEventListener) window.addEventListener("resize", app_onresize, false);
@@ -35,12 +42,13 @@ function app_init() {
   if (app_is_ie) {
     document.onkeydown = app_onkeydown;
     document.getElementById('imageview').style.position = 'absolute';
-    document.getElementById('popup_details').style.position = 'absolute';
+    popup.style.position = 'absolute';
     document.getElementById('popup_titlebar').style.position = 'absolute';
   } else if (app_is_safari) {
     document.getElementById('tools_right').style.paddingRight = '8px';
   }
 
+  app_onresize();
   var i = app_getcookie();
   if (i >= 0) image_show(i);
 }
@@ -69,7 +77,7 @@ function app_setcookie() {
   var d = new Date(), c = slide_order + ';' + (slide_delay/1000) + ';' + sidebar_on + ';' +
     album_detailson + ';' + album_itemlinkson + ';' + text_on + ';';
   d.setTime(d.getTime() + 90*24*60*60*1000); // 90 day cookie
-  document.cookie = 'G2_hybrid=' + escape(c) + ';expires=' + d.toUTCString();
+  document.cookie = 'G2_hybrid=' + escape(c) + ';path=' + cookie_path + ';expires=' + d.toUTCString();
 }
 function app_getcookie() {
   var c = getcookie('G2_hybrid'), i,j,v,n,it=1,r=-1;
@@ -80,9 +88,9 @@ function app_getcookie() {
       switch (it++) {
 	case 1: ui_select('slide_order', n); slide_setorder(n); break;
 	case 2: ui_select('slide_delay', n); slide_setdelay(n); break;
-	case 3: if (!n) sidebar_onoff(); break;
+	case 3: if (n) sidebar_onoff(); break;
 	case 4: if (!n) album_detailsonoff(); break;
-	case 5: if (!n) album_itemlinksonoff(); break;
+	case 5: if (n) album_itemlinksonoff(); break;
 	case 6: if (n) text_onoff(); break;
       }
     }
@@ -105,6 +113,8 @@ function app_onkeypress(event) {
     case 63235: keyCode=39; break;  case 63277: keyCode=34; break;
   }
   /* Album view: space = start slideshow
+   *             ctrl-right/left = show/hide sidebar
+   *             ctrl-up/down = show/hide item links
    * Image view: space = start/pause slideshow
    *             escape = return to album view
    *             left/right = next/prev image
@@ -116,10 +126,24 @@ function app_onkeypress(event) {
    */
   if (keyCode==32) slide_onoff();
   else if (keyCode==27) { if (popup_on) popup_vis(0); else if (image_on) image_vis(0); }
-  if (!image_on || keyCode < 33 || keyCode > 40) return;
+  if (keyCode < 33 || keyCode > 40) return;
   if (event.shiftKey) keyCode += 100;
   if (event.ctrlKey) keyCode += 200;
-  switch (keyCode) {
+  if (!image_on) switch (keyCode) {
+    case 239: //Ctrl-Right
+      if (!sidebar_on) sidebar_onoff();
+      break;
+    case 237: //Ctrl-Left
+      if (sidebar_on) sidebar_onoff();
+      break;
+    case 238: //Ctrl-Up
+      if (!album_itemlinkson) album_itemlinksonoff();
+      break;
+    case 240: //Ctrl-Down
+      if (album_itemlinkson) album_itemlinksonoff();
+      break;
+  }
+  else switch (keyCode) {
     case 37: //Left
       if (image_zoomon) { imagearea.scrollLeft -= 20; break; }
     case 137: //Shift-Left
@@ -163,7 +187,7 @@ function app_onkeydown() {
 
 //Class album :: gsContent(album_titlebar(album_tools,album_desc,album_info),gsAlbumContent)
 var album_detailson=1, // Details are visible
-    album_itemlinkson=1, // Item links are visible
+    album_itemlinkson=0, // Item links are visible
     album_fixedtitle=0; // Using fixed position for album_titlebar
 function album_detailsonoff() {
   ui_vis('album_info', (album_detailson = album_detailson?0:1));
@@ -176,7 +200,7 @@ function album_itemlinksonoff() {
   album_itemlinkson = album_itemlinkson?0:1;
   for (var i = 0; i < imgs.length; i++)
     if (imgs[i].className == 'popup_button')
-      imgs[i].style.visibility = album_itemlinkson ? 'visible' : 'hidden';
+      imgs[i].style.display = album_itemlinkson ? 'inline' : 'none';
   ui_sethtml('lnk_link', album_itemlinkson ? album_hidelinks : album_showlinks);
 }
 function album_setfixedtitle() {
@@ -194,7 +218,7 @@ function album_setmargin() {
 }
 
 //Class sidebar :: div sidebar
-var sidebar_on=1; // Sidebar is visible
+var sidebar_on=0; // Sidebar is visible
 function sidebar_onoff() {
   ui_vis('sidebar_max', sidebar_on, 1);
   ui_vis('sidebar_min', (sidebar_on = sidebar_on?0:1), 1);
@@ -208,7 +232,8 @@ var image_on=0, // Image is visible
     image_cache = new Image, // For precaching an image
     image_iscached = new Array(data_count), // Track precached images
     imagearea, imagediv, textdiv, // Containers
-    text_on=0; // Description text is visible
+    text_on=0, // Description text is visible
+    text_empty=0; // Description text is empty
 function image_setsize() {
   imagearea.style.height = (app_wh - textdiv.offsetHeight) + 'px';
 }
@@ -234,6 +259,7 @@ function image_show(i) {
   slide_reset();
   image_index = i;
   ui_sethtml('title', document.getElementById('title_'+image_index).innerHTML);
+  ui_sethtml('date', document.getElementById('date_'+image_index).innerHTML);
   image_setsize();
   if (data_iw[i] < 0) {
     imagediv.innerHTML = '<iframe style="width:100%;height:' + (imagearea.offsetHeight - 4)
@@ -292,7 +318,8 @@ function image_precache(i) {
   }
 }
 function image_loaded() {
-  var i = slide_nextindex(); if (i >= 0) image_precache(i);
+  var i = slide_nextindex(); if (i < 0 || image_iscached[i]) i = slide_previndex();
+  if (i >= 0) image_precache(i);
   slide_go(i);
 }
 function image_next() {
@@ -307,10 +334,15 @@ function image_setbuttons() {
   ui_vis('next_off', i < 0, 1);
   ui_vis('prev_img', j >= 0, 1);
   ui_vis('prev_off', j < 0, 1);
+  text_empty = document.getElementById('text_'+image_index).innerHTML ?0:1;
+  if (!text_on) {
+    ui_vis('text_on', !text_empty, 1);
+    ui_vis('text_none', text_empty, 1);
+  }
 }
 function text_onoff() {
   if ((text_on = text_on?0:1) && data_count > 0) text_fill();
-  ui_vis('text_on', !text_on, 1);
+  ui_vis(text_empty ? 'text_none' : 'text_on', !text_on, 1);
   ui_vis('text_off', text_on, 1);
   ui_vis('text', text_on);
   if (image_on) { image_setsize(); image_fit(); }
@@ -366,6 +398,7 @@ function slide_fillrandom(lockfirst) {
   }
 }
 function slide_setbuttons() {
+  if (!data_count) return;
   ui_vis('slide_poz', slide_on, 1);
   ui_vis('slide_fwd', (!slide_on && slide_order > 0), 1);
   ui_vis('slide__fwd', (slide_order > 0), 1);
@@ -433,13 +466,9 @@ function popup_menu(event,i,ii) {
   pop.style.visibility = 'visible';
 }
 function popup_info(i) {
-  if (window.frames.length) {
-    window.frames[0].document.location.replace(
-      document.getElementById('info_' + (i >= 0 ? i : image_index)).href);
-  } else { //For Safari
-    document.getElementById('popup_details').firstChild.src =
-      document.getElementById('info_' + (i >= 0 ? i : image_index)).href;
-  }
+  var o = document.getElementById('popup_details').firstChild,
+      href = document.getElementById('info_' + (i >= 0 ? i : image_index)).href;
+  o.src = href;  // See app_init; use o.data = href if browsers ever support object.
   popup_vis(1);
 }
 function popup_vis(on) {
